<?php

namespace ZRay;

class Aws
{
	public function collect(array $context, array &$storage)
	{
		$awsUrlLatest = "http://169.254.169.254/latest/";

		$cacheFileName = sys_get_temp_dir() . '/zray_aws_data';
		$awsData = [];
		try {
			$ts = filemtime($cacheFileName);
			if ($ts < time() - 3600) {
				$awsData = file_get_contents($cacheFileName);
			}
			$awsData = json_decode(file_get_contents($cacheFileName), true);
		} catch (\Exception $th) {
		}
		if (!$awsData) {
			$awsData = [
				'meta' => $this->getMetaData($awsUrlLatest . "meta-data"),
				'user' => $this->getData($awsUrlLatest . "user-data")
			];
		}
		if ($awsData) {
			file_put_contents($cacheFileName, json_encode($awsData), LOCK_EX);
		}

		if ($awsData['meta']) {
			$storage['Meta data'][] = $awsData['meta'];
		}
		if ($awsData['user']) {
			$storage['User data'][] = $awsData['user'];
		}
	}

	public static function shutdown(): bool
	{
		return true;
	}

	private function getMetaData($url)
	{
		$result = [];
		$data = $this->getData($url);
		if ($data) {
			foreach (explode("\n", $data) as $name) {
				$name = strstr($name, '=', true) == '' ? $name : strstr($name, '=', true) . '/';
				// omitting sensitive data
				if (in_array($name, ['identity-credentials/', 'public-keys/'])) {
					continue;
				}
				if (strstr($name, '/')) {
					$result[$name] = $this->getMetaData("{$url}/{$name}");
				}
				elseif ($name !== "") {
					$result[$name] = $this->getData("{$url}/{$name}");
				}
			}
		}
		return $result;
	}

	private function getData(string $url)
	{
		$body = '';
		try {
			$body = file_get_contents($url);
		} catch (\Throwable $th) {
		}
		if ($body) {
			try {
				$data = json_decode($body, true);
				if ($data !== null) {
					$body = $data;
				}
			} catch (\Throwable $e) {
			}
		}

		return $body;
	}
}

if (class_exists('ZRayPlugin', false) && function_exists('register_shutdown_function')) {

	register_shutdown_function('ZRay\Aws::shutdown');

	$zrp = new \ZRayPlugin("AWS", true);

	$aws = new Aws();

	$zrp->traceFunction(
		'ZRay\Aws::shutdown',
		null,
		[$aws, 'collect']
	);
}